# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ..yes_no_indicator.types.yes_no_indicator import YesNoIndicator
from .errors.diagnosis_not_found_http_error import DiagnosisNotFoundHttpError
from .errors.disallow_multiple_primary_diagnosis_http_error import DisallowMultiplePrimaryDiagnosisHttpError
from .errors.service_lines_must_have_at_least_one_diagnosis_http_error import (
    ServiceLinesMustHaveAtLeastOneDiagnosisHttpError,
)
from .types.diagnosis import Diagnosis
from .types.diagnosis_id import DiagnosisId
from .types.diagnosis_not_found_error import DiagnosisNotFoundError
from .types.diagnosis_type_code import DiagnosisTypeCode
from .types.disallow_multiple_primary_diagnosis_error import DisallowMultiplePrimaryDiagnosisError
from .types.service_lines_must_have_at_least_one_diagnosis_error import ServiceLinesMustHaveAtLeastOneDiagnosisError
from .types.standalone_diagnosis_create import StandaloneDiagnosisCreate

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDiagnosesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self, *, request: StandaloneDiagnosisCreate, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Diagnosis]:
        """
        Creates a new diagnosis for an encounter

        Parameters
        ----------
        request : StandaloneDiagnosisCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Diagnosis]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/diagnoses/v2",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Diagnosis,
                parse_obj_as(
                    type_=Diagnosis,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        diagnosis_id: DiagnosisId,
        *,
        name: typing.Optional[str] = OMIT,
        code_type: typing.Optional[DiagnosisTypeCode] = OMIT,
        code: typing.Optional[str] = OMIT,
        present_on_admission_indicator: typing.Optional[YesNoIndicator] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Diagnosis]:
        """
        Updates the diagnosis record matching the provided `diagnosis_id`

        Parameters
        ----------
        diagnosis_id : DiagnosisId

        name : typing.Optional[str]
            Empty string not allowed.

        code_type : typing.Optional[DiagnosisTypeCode]
            Typically, providers submitting claims to Candid are using ICD-10 diagnosis codes. If you are using ICD-10 codes, the primary diagnosis code listed on the claim should use the ABK code_type. If more than one diagnosis is being submitted on a claim, please use ABF for the rest of the listed diagnoses. If you are using ICD-9 diagnosis codes, use BK and BF for the principal and following diagnosis code(s) respectively.

        code : typing.Optional[str]
            Empty string not allowed.
            Should be of the appropriate format for the provided `code_type`.
            Must obey the ICD-10 format if an ICD-10 code_type is provided, specifically:
              - Letter
              - Digit
              - Digit or the letter `A` or `B`
              - (Optional) Period `.`
              - Up to 4 (or as few as 0) letters and digits

        present_on_admission_indicator : typing.Optional[YesNoIndicator]
            For Institutional claims only.
            A "Y" indicates that the onset occurred prior to admission to the hospital.
            An "N" indicates that the onset did NOT occur prior to admission to the hospital.
            A "U" indicates that it is unknown whether the onset occurred prior to admission to the hospital or not.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Diagnosis]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/diagnoses/v2/{jsonable_encoder(diagnosis_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json={
                "name": name,
                "code_type": code_type,
                "code": code,
                "present_on_admission_indicator": present_on_admission_indicator,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Diagnosis,
                parse_obj_as(
                    type_=Diagnosis,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "DiagnosisNotFoundHTTPError":
                raise DiagnosisNotFoundHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        DiagnosisNotFoundError,
                        parse_obj_as(
                            type_=DiagnosisNotFoundError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "DisallowMultiplePrimaryDiagnosisHTTPError":
                raise DisallowMultiplePrimaryDiagnosisHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        DisallowMultiplePrimaryDiagnosisError,
                        parse_obj_as(
                            type_=DisallowMultiplePrimaryDiagnosisError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, diagnosis_id: DiagnosisId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Deletes the diagnosis record associated with the provided `diagnosis_id`

        Parameters
        ----------
        diagnosis_id : DiagnosisId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/diagnoses/v2/{jsonable_encoder(diagnosis_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return HttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "DiagnosisNotFoundHTTPError":
                raise DiagnosisNotFoundHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        DiagnosisNotFoundError,
                        parse_obj_as(
                            type_=DiagnosisNotFoundError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "ServiceLinesMustHaveAtLeastOneDiagnosisHTTPError":
                raise ServiceLinesMustHaveAtLeastOneDiagnosisHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ServiceLinesMustHaveAtLeastOneDiagnosisError,
                        parse_obj_as(
                            type_=ServiceLinesMustHaveAtLeastOneDiagnosisError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawDiagnosesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self, *, request: StandaloneDiagnosisCreate, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Diagnosis]:
        """
        Creates a new diagnosis for an encounter

        Parameters
        ----------
        request : StandaloneDiagnosisCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Diagnosis]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/diagnoses/v2",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Diagnosis,
                parse_obj_as(
                    type_=Diagnosis,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        diagnosis_id: DiagnosisId,
        *,
        name: typing.Optional[str] = OMIT,
        code_type: typing.Optional[DiagnosisTypeCode] = OMIT,
        code: typing.Optional[str] = OMIT,
        present_on_admission_indicator: typing.Optional[YesNoIndicator] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Diagnosis]:
        """
        Updates the diagnosis record matching the provided `diagnosis_id`

        Parameters
        ----------
        diagnosis_id : DiagnosisId

        name : typing.Optional[str]
            Empty string not allowed.

        code_type : typing.Optional[DiagnosisTypeCode]
            Typically, providers submitting claims to Candid are using ICD-10 diagnosis codes. If you are using ICD-10 codes, the primary diagnosis code listed on the claim should use the ABK code_type. If more than one diagnosis is being submitted on a claim, please use ABF for the rest of the listed diagnoses. If you are using ICD-9 diagnosis codes, use BK and BF for the principal and following diagnosis code(s) respectively.

        code : typing.Optional[str]
            Empty string not allowed.
            Should be of the appropriate format for the provided `code_type`.
            Must obey the ICD-10 format if an ICD-10 code_type is provided, specifically:
              - Letter
              - Digit
              - Digit or the letter `A` or `B`
              - (Optional) Period `.`
              - Up to 4 (or as few as 0) letters and digits

        present_on_admission_indicator : typing.Optional[YesNoIndicator]
            For Institutional claims only.
            A "Y" indicates that the onset occurred prior to admission to the hospital.
            An "N" indicates that the onset did NOT occur prior to admission to the hospital.
            A "U" indicates that it is unknown whether the onset occurred prior to admission to the hospital or not.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Diagnosis]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/diagnoses/v2/{jsonable_encoder(diagnosis_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json={
                "name": name,
                "code_type": code_type,
                "code": code,
                "present_on_admission_indicator": present_on_admission_indicator,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Diagnosis,
                parse_obj_as(
                    type_=Diagnosis,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "DiagnosisNotFoundHTTPError":
                raise DiagnosisNotFoundHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        DiagnosisNotFoundError,
                        parse_obj_as(
                            type_=DiagnosisNotFoundError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "DisallowMultiplePrimaryDiagnosisHTTPError":
                raise DisallowMultiplePrimaryDiagnosisHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        DisallowMultiplePrimaryDiagnosisError,
                        parse_obj_as(
                            type_=DisallowMultiplePrimaryDiagnosisError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, diagnosis_id: DiagnosisId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Deletes the diagnosis record associated with the provided `diagnosis_id`

        Parameters
        ----------
        diagnosis_id : DiagnosisId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/diagnoses/v2/{jsonable_encoder(diagnosis_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return AsyncHttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "DiagnosisNotFoundHTTPError":
                raise DiagnosisNotFoundHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        DiagnosisNotFoundError,
                        parse_obj_as(
                            type_=DiagnosisNotFoundError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "ServiceLinesMustHaveAtLeastOneDiagnosisHTTPError":
                raise ServiceLinesMustHaveAtLeastOneDiagnosisHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ServiceLinesMustHaveAtLeastOneDiagnosisError,
                        parse_obj_as(
                            type_=ServiceLinesMustHaveAtLeastOneDiagnosisError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.unauthorized_error import UnauthorizedError
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.schema_id import SchemaId
from ....commons.types.unauthorized_error_message import UnauthorizedErrorMessage
from .errors.schema_validation_http_failure import SchemaValidationHttpFailure
from .types.schema import Schema
from .types.schema_field import SchemaField
from .types.schema_get_multi_response import SchemaGetMultiResponse
from .types.schema_validation_failure import SchemaValidationFailure

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_multi(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SchemaGetMultiResponse]:
        """
        Returns all custom schemas.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SchemaGetMultiResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/custom-schemas/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                SchemaGetMultiResponse,
                parse_obj_as(
                    type_=SchemaGetMultiResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, schema_id: SchemaId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Schema]:
        """
        Return a custom schema with a given ID.

        Parameters
        ----------
        schema_id : SchemaId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Schema]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/custom-schemas/v1/{jsonable_encoder(schema_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Schema,
                parse_obj_as(
                    type_=Schema,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        name: str,
        fields: typing.Sequence[SchemaField],
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Schema]:
        """
        Create custom schema with a set of typed keys. Schema keys can be referenced as inputs in user-configurable rules in the Rules
        Engine, and key-value pairs can be attached to claims via the Encounters API.

        Parameters
        ----------
        name : str

        fields : typing.Sequence[SchemaField]

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Schema]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/custom-schemas/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json={
                "name": name,
                "description": description,
                "fields": fields,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Schema,
                parse_obj_as(
                    type_=Schema,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "SchemaValidationHttpFailure":
                raise SchemaValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaValidationFailure,
                        parse_obj_as(
                            type_=SchemaValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        schema_id: SchemaId,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        fields_to_add: typing.Optional[typing.Sequence[SchemaField]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Schema]:
        """
        Update the name, description, or keys on a preexisting schema.

        Parameters
        ----------
        schema_id : SchemaId

        name : typing.Optional[str]

        description : typing.Optional[str]

        fields_to_add : typing.Optional[typing.Sequence[SchemaField]]
            A list of typed entries to add to schema. Only additive modifications are permitted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Schema]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/custom-schemas/v1/{jsonable_encoder(schema_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json={
                "name": name,
                "description": description,
                "fields_to_add": fields_to_add,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Schema,
                parse_obj_as(
                    type_=Schema,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "SchemaValidationHttpFailure":
                raise SchemaValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaValidationFailure,
                        parse_obj_as(
                            type_=SchemaValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_multi(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SchemaGetMultiResponse]:
        """
        Returns all custom schemas.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SchemaGetMultiResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/custom-schemas/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                SchemaGetMultiResponse,
                parse_obj_as(
                    type_=SchemaGetMultiResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, schema_id: SchemaId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Schema]:
        """
        Return a custom schema with a given ID.

        Parameters
        ----------
        schema_id : SchemaId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Schema]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/custom-schemas/v1/{jsonable_encoder(schema_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Schema,
                parse_obj_as(
                    type_=Schema,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        name: str,
        fields: typing.Sequence[SchemaField],
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Schema]:
        """
        Create custom schema with a set of typed keys. Schema keys can be referenced as inputs in user-configurable rules in the Rules
        Engine, and key-value pairs can be attached to claims via the Encounters API.

        Parameters
        ----------
        name : str

        fields : typing.Sequence[SchemaField]

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Schema]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/custom-schemas/v1",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json={
                "name": name,
                "description": description,
                "fields": fields,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Schema,
                parse_obj_as(
                    type_=Schema,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "SchemaValidationHttpFailure":
                raise SchemaValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaValidationFailure,
                        parse_obj_as(
                            type_=SchemaValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        schema_id: SchemaId,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        fields_to_add: typing.Optional[typing.Sequence[SchemaField]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Schema]:
        """
        Update the name, description, or keys on a preexisting schema.

        Parameters
        ----------
        schema_id : SchemaId

        name : typing.Optional[str]

        description : typing.Optional[str]

        fields_to_add : typing.Optional[typing.Sequence[SchemaField]]
            A list of typed entries to add to schema. Only additive modifications are permitted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Schema]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/custom-schemas/v1/{jsonable_encoder(schema_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json={
                "name": name,
                "description": description,
                "fields_to_add": fields_to_add,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Schema,
                parse_obj_as(
                    type_=Schema,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "SchemaValidationHttpFailure":
                raise SchemaValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaValidationFailure,
                        parse_obj_as(
                            type_=SchemaValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.service_line_id import ServiceLineId
from .insurance_adjustment_info import InsuranceAdjustmentInfo
from .insurance_payment_info import InsurancePaymentInfo
from .non_insurance_adjustment_info import NonInsuranceAdjustmentInfo
from .non_insurance_payment_info import NonInsurancePaymentInfo
from .patient_adjustment_info import PatientAdjustmentInfo
from .patient_payment_info import PatientPaymentInfo


class ServiceLineItemization(UniversalBaseModel):
    service_line_id: ServiceLineId = pydantic.Field()
    """
    The ID of the service line.
    """

    procedure_code: str = pydantic.Field()
    """
    The procedure code (CPT/HCPCS) associated with the service line.
    """

    date_of_service: dt.date = pydantic.Field()
    """
    The date of service for the service line.
    """

    patient_balance_cents: int = pydantic.Field()
    """
    The total patient balance in cents for the service line. Negative values indicate a credit balance.
    """

    charge_amount_cents: int = pydantic.Field()
    """
    The total charge amount in cents for the service line.
    """

    insurance_adjustments: InsuranceAdjustmentInfo
    insurance_payments: InsurancePaymentInfo
    non_insurance_adjustments: NonInsuranceAdjustmentInfo
    non_insurance_payments: NonInsurancePaymentInfo
    patient_adjustments: PatientAdjustmentInfo
    patient_payments: PatientPaymentInfo
    copay_cents: int = pydantic.Field()
    """
    The copay amount in cents for the service line.
    """

    coinsurance_cents: int = pydantic.Field()
    """
    The coinsurance amount in cents for the service line.
    """

    deductible_cents: int = pydantic.Field()
    """
    The deductible amount in cents for the service line.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2
from .....common.types.base_model import BaseModel
from .image_id import ImageId
from .mutable_image import MutableImage


class Image(BaseModel, MutableImage):
    """
    An Image object with immutable server-owned properties.
    """

    id: ImageId
    signed_url: str = pydantic.Field()
    """
    A signed URL to the image.  This url can be used to upload an image to GCP storage or to read the image contents.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2
from .....common.types.base_model import BaseModel
from .....common.types.patient_id import PatientId
from .mutable_patient import MutablePatient


class Patient(BaseModel, MutablePatient):
    """
    A patient object with immutable server-owned properties.
    """

    id: PatientId = pydantic.Field()
    """
    The unique UUID identifier for a Patient.  Patient ID is used in machine contexts.
    """

    mrn: str = pydantic.Field()
    """
    The medical record number for the patient.  Human-friendly Candid generated MRNs are of the form "YYMMDDXXXX", where "YYMMDD" is the date of patient creation and "XXXX" is a zero-padded incrementing integer.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.date import Date
from ...v_2.types.authorized_signatory_update import AuthorizedSignatoryUpdate
from ...v_2.types.contract_status import ContractStatus
from ...v_2.types.date_update import DateUpdate
from ...v_2.types.insurance_types import InsuranceTypes
from ...v_2.types.regions_update import RegionsUpdate
from .contract_service_facility_id import ContractServiceFacilityId
from .rendering_providerid import RenderingProviderid


class ContractUpdateUnion_Professional(UniversalBaseModel):
    type: typing.Literal["professional"] = "professional"
    rendering_provider_ids: typing.Optional[typing.Set[RenderingProviderid]] = None
    effective_date: typing.Optional[Date] = None
    expiration_date: typing.Optional[DateUpdate] = None
    regions: typing.Optional[RegionsUpdate] = None
    contract_status: typing.Optional[ContractStatus] = None
    authorized_signatory: typing.Optional[AuthorizedSignatoryUpdate] = None
    commercial_insurance_types: typing.Optional[InsuranceTypes] = None
    medicare_insurance_types: typing.Optional[InsuranceTypes] = None
    medicaid_insurance_types: typing.Optional[InsuranceTypes] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ContractUpdateUnion_Institutional(UniversalBaseModel):
    type: typing.Literal["institutional"] = "institutional"
    contract_service_facility_ids: typing.Optional[typing.Set[ContractServiceFacilityId]] = None
    effective_date: typing.Optional[Date] = None
    expiration_date: typing.Optional[DateUpdate] = None
    regions: typing.Optional[RegionsUpdate] = None
    contract_status: typing.Optional[ContractStatus] = None
    authorized_signatory: typing.Optional[AuthorizedSignatoryUpdate] = None
    commercial_insurance_types: typing.Optional[InsuranceTypes] = None
    medicare_insurance_types: typing.Optional[InsuranceTypes] = None
    medicaid_insurance_types: typing.Optional[InsuranceTypes] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ContractUpdateUnion = typing_extensions.Annotated[
    typing.Union[ContractUpdateUnion_Professional, ContractUpdateUnion_Institutional],
    pydantic.Field(discriminator="type"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .admitting_diagnosis_new import AdmittingDiagnosisNew
from .condition_information_new import ConditionInformationNew
from .diagnosis_related_group_new import DiagnosisRelatedGroupNew
from .external_cause_of_injury_new import ExternalCauseOfInjuryNew
from .occurrence_information_new import OccurrenceInformationNew
from .occurrence_span_information_new import OccurrenceSpanInformationNew
from .other_diagnosis_information_new import OtherDiagnosisInformationNew
from .other_procedure_information_new import OtherProcedureInformationNew
from .patient_reason_for_visit_new import PatientReasonForVisitNew
from .principal_diagnosis_new import PrincipalDiagnosisNew
from .principal_procedure_information_new import PrincipalProcedureInformationNew
from .treatment_code_information_new import TreatmentCodeInformationNew
from .value_information_new import ValueInformationNew


class HealthCareCodeInformationCreate(UniversalBaseModel):
    principal_diagnosis: typing.Optional[PrincipalDiagnosisNew] = None
    other_diagnosis_information: typing.Optional[typing.List[OtherDiagnosisInformationNew]] = None
    admitting_diagnosis: typing.Optional[AdmittingDiagnosisNew] = pydantic.Field(default=None)
    """
    This object only applies to 837i institutional claim forms.
    """

    patient_reasons_for_visit: typing.Optional[typing.List[PatientReasonForVisitNew]] = pydantic.Field(default=None)
    """
    This object only applies to 837i institutional claim forms.
    """

    external_causes_of_injury: typing.Optional[typing.List[ExternalCauseOfInjuryNew]] = pydantic.Field(default=None)
    """
    This object only applies to 837i institutional claim forms.
    """

    diagnosis_related_groups: typing.Optional[DiagnosisRelatedGroupNew] = pydantic.Field(default=None)
    """
    This object only applies to 837i institutional claim forms.
    """

    principal_procedure: typing.Optional[PrincipalProcedureInformationNew] = pydantic.Field(default=None)
    """
    This object only applies to 837i institutional claim forms.
    """

    other_procedure_information: typing.Optional[typing.List[OtherProcedureInformationNew]] = pydantic.Field(
        default=None
    )
    """
    This object only applies to 837i institutional claim forms.
    """

    occurrence_span_information: typing.Optional[typing.List[OccurrenceSpanInformationNew]] = pydantic.Field(
        default=None
    )
    """
    This object only applies to 837i institutional claim forms.
    """

    occurrence_information: typing.Optional[typing.List[OccurrenceInformationNew]] = pydantic.Field(default=None)
    """
    This object only applies to 837i institutional claim forms.
    """

    treatment_code_information: typing.Optional[typing.List[TreatmentCodeInformationNew]] = pydantic.Field(default=None)
    """
    This object only applies to 837i institutional claim forms.
    """

    value_information: typing.Optional[typing.List[ValueInformationNew]] = pydantic.Field(default=None)
    """
    This object only applies to 837i institutional claim forms.
    """

    condition_information: typing.Optional[typing.List[ConditionInformationNew]] = pydantic.Field(default=None)
    """
    This object only applies to 837i institutional claim forms.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        CardPaymentMethod,
        CardPaymentMethodCreate,
        CashPaymentMethod,
        CashPaymentMethodCreate,
        CheckPaymentMethod,
        CheckPaymentMethodCreate,
        MoneyOrderPaymentMethod,
        MoneyOrderPaymentMethodCreate,
        PatientPayment,
        PatientPaymentId,
        PatientPaymentSortField,
        PatientPaymentsPage,
        PaymentMethod,
        PaymentMethodCreate,
        PaymentMethodCreate_Card,
        PaymentMethodCreate_Cash,
        PaymentMethodCreate_Check,
        PaymentMethodCreate_MoneyOrder,
        PaymentMethodDetail,
        PaymentMethodDetailCreate,
        PaymentMethod_Card,
        PaymentMethod_Cash,
        PaymentMethod_Check,
        PaymentMethod_MoneyOrder,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "CardPaymentMethod": ".types",
    "CardPaymentMethodCreate": ".types",
    "CashPaymentMethod": ".types",
    "CashPaymentMethodCreate": ".types",
    "CheckPaymentMethod": ".types",
    "CheckPaymentMethodCreate": ".types",
    "MoneyOrderPaymentMethod": ".types",
    "MoneyOrderPaymentMethodCreate": ".types",
    "PatientPayment": ".types",
    "PatientPaymentId": ".types",
    "PatientPaymentSortField": ".types",
    "PatientPaymentsPage": ".types",
    "PaymentMethod": ".types",
    "PaymentMethodCreate": ".types",
    "PaymentMethodCreate_Card": ".types",
    "PaymentMethodCreate_Cash": ".types",
    "PaymentMethodCreate_Check": ".types",
    "PaymentMethodCreate_MoneyOrder": ".types",
    "PaymentMethodDetail": ".types",
    "PaymentMethodDetailCreate": ".types",
    "PaymentMethod_Card": ".types",
    "PaymentMethod_Cash": ".types",
    "PaymentMethod_Check": ".types",
    "PaymentMethod_MoneyOrder": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "CardPaymentMethod",
    "CardPaymentMethodCreate",
    "CashPaymentMethod",
    "CashPaymentMethodCreate",
    "CheckPaymentMethod",
    "CheckPaymentMethodCreate",
    "MoneyOrderPaymentMethod",
    "MoneyOrderPaymentMethodCreate",
    "PatientPayment",
    "PatientPaymentId",
    "PatientPaymentSortField",
    "PatientPaymentsPage",
    "PaymentMethod",
    "PaymentMethodCreate",
    "PaymentMethodCreate_Card",
    "PaymentMethodCreate_Cash",
    "PaymentMethodCreate_Check",
    "PaymentMethodCreate_MoneyOrder",
    "PaymentMethodDetail",
    "PaymentMethodDetailCreate",
    "PaymentMethod_Card",
    "PaymentMethod_Cash",
    "PaymentMethod_Check",
    "PaymentMethod_MoneyOrder",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        BillingProvider,
        BillingProviderSecondaryIdentification,
        BillingProviderSecondaryIdentificationQualifier,
        BillingProviderUpdate,
        BillingProviderUpdateWithOptionalAddress,
        EncounterProvider,
        EncounterProviderBase,
        InitialReferringProvider,
        InitialReferringProviderUpdate,
        InitialReferringProviderUpdateWithOptionalAddress,
        OrderingProvider,
        OrderingProviderOptional,
        OrderingProviderUpdate,
        ProviderId,
        ProviderSecondaryIdentification,
        ProviderSecondaryIdentificationQualifier,
        ReferringProvider,
        ReferringProviderSecondaryIdentification,
        ReferringProviderSecondaryIdentificationQualifier,
        ReferringProviderUpdate,
        ReferringProviderUpdateWithOptionalAddress,
        RenderingProvider,
        RenderingProviderUpdate,
        RenderingProviderUpdateWithOptionalAddress,
        SupervisingProvider,
        SupervisingProviderSecondaryIdentification,
        SupervisingProviderUpdate,
        SupervisingProviderUpdateWithOptionalAddress,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "BillingProvider": ".types",
    "BillingProviderSecondaryIdentification": ".types",
    "BillingProviderSecondaryIdentificationQualifier": ".types",
    "BillingProviderUpdate": ".types",
    "BillingProviderUpdateWithOptionalAddress": ".types",
    "EncounterProvider": ".types",
    "EncounterProviderBase": ".types",
    "InitialReferringProvider": ".types",
    "InitialReferringProviderUpdate": ".types",
    "InitialReferringProviderUpdateWithOptionalAddress": ".types",
    "OrderingProvider": ".types",
    "OrderingProviderOptional": ".types",
    "OrderingProviderUpdate": ".types",
    "ProviderId": ".types",
    "ProviderSecondaryIdentification": ".types",
    "ProviderSecondaryIdentificationQualifier": ".types",
    "ReferringProvider": ".types",
    "ReferringProviderSecondaryIdentification": ".types",
    "ReferringProviderSecondaryIdentificationQualifier": ".types",
    "ReferringProviderUpdate": ".types",
    "ReferringProviderUpdateWithOptionalAddress": ".types",
    "RenderingProvider": ".types",
    "RenderingProviderUpdate": ".types",
    "RenderingProviderUpdateWithOptionalAddress": ".types",
    "SupervisingProvider": ".types",
    "SupervisingProviderSecondaryIdentification": ".types",
    "SupervisingProviderUpdate": ".types",
    "SupervisingProviderUpdateWithOptionalAddress": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BillingProvider",
    "BillingProviderSecondaryIdentification",
    "BillingProviderSecondaryIdentificationQualifier",
    "BillingProviderUpdate",
    "BillingProviderUpdateWithOptionalAddress",
    "EncounterProvider",
    "EncounterProviderBase",
    "InitialReferringProvider",
    "InitialReferringProviderUpdate",
    "InitialReferringProviderUpdateWithOptionalAddress",
    "OrderingProvider",
    "OrderingProviderOptional",
    "OrderingProviderUpdate",
    "ProviderId",
    "ProviderSecondaryIdentification",
    "ProviderSecondaryIdentificationQualifier",
    "ReferringProvider",
    "ReferringProviderSecondaryIdentification",
    "ReferringProviderSecondaryIdentificationQualifier",
    "ReferringProviderUpdate",
    "ReferringProviderUpdateWithOptionalAddress",
    "RenderingProvider",
    "RenderingProviderUpdate",
    "RenderingProviderUpdateWithOptionalAddress",
    "SupervisingProvider",
    "SupervisingProviderSecondaryIdentification",
    "SupervisingProviderUpdate",
    "SupervisingProviderUpdateWithOptionalAddress",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from .......core.api_error import ApiError
from .......core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .......core.datetime_utils import serialize_datetime
from .......core.http_response import AsyncHttpResponse, HttpResponse
from .......core.jsonable_encoder import jsonable_encoder
from .......core.pydantic_utilities import parse_obj_as
from .......core.request_options import RequestOptions
from ....common.errors.not_found_error import NotFoundError
from ....common.errors.version_conflict_error import VersionConflictError
from ....common.types.error_base_4_xx import ErrorBase4Xx
from ....common.types.filter_query_string import FilterQueryString
from ....common.types.page_token import PageToken
from ....common.types.version_conflict_error_body import VersionConflictErrorBody
from .types.batch_eligibility_response import BatchEligibilityResponse
from .types.eligibility_check_page import EligibilityCheckPage
from .types.eligibility_recommendation import EligibilityRecommendation
from .types.eligibility_request import EligibilityRequest
from .types.eligibility_response import EligibilityResponse
from .types.payer_search_response import PayerSearchResponse
from .types.post_eligibility_recommendation_request import PostEligibilityRecommendationRequest
from .types.vote import Vote

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def post(
        self, *, request: EligibilityRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[EligibilityResponse]:
        """
        Sends real-time eligibility checks to payers through Stedi.

        Parameters
        ----------
        request : EligibilityRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EligibilityResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "eligibility-checks/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EligibilityResponse,
                parse_obj_as(
                    type_=EligibilityResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def batch(
        self, *, request: typing.Sequence[EligibilityRequest], request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[BatchEligibilityResponse]:
        """
        Sends a batch of eligibility checks to payers through Stedi.

        Parameters
        ----------
        request : typing.Sequence[EligibilityRequest]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BatchEligibilityResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "eligibility-checks/v1/batch",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                BatchEligibilityResponse,
                parse_obj_as(
                    type_=BatchEligibilityResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def poll_batch(
        self,
        batch_id: str,
        *,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EligibilityCheckPage]:
        """
        Polls the status of a batch eligibility check.

        Parameters
        ----------
        batch_id : str

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EligibilityCheckPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"eligibility-checks/v1/batch/{jsonable_encoder(batch_id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EligibilityCheckPage,
                parse_obj_as(
                    type_=EligibilityCheckPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def payer_search(
        self,
        *,
        page_size: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        query: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayerSearchResponse]:
        """
        Searches for payers that match the query parameters.

        Parameters
        ----------
        page_size : typing.Optional[int]

        page_token : typing.Optional[PageToken]

        query : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayerSearchResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "eligibility-checks/v1/payer/search",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "page_size": page_size,
                "page_token": page_token,
                "query": query,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PayerSearchResponse,
                parse_obj_as(
                    type_=PayerSearchResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def recommendation(
        self,
        *,
        filters: typing.Optional[FilterQueryString] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.List[EligibilityRecommendation]]:
        """
        Gets recommendation for eligibility checks based on filters. This endpoint will retrieve all the latest eligibility recommendations for each
        eligibility recommendation type for the given filters. If you want to get a specific recommendation type, you can use the `type` query parameter.

        Parameters
        ----------
        filters : typing.Optional[FilterQueryString]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[EligibilityRecommendation]]
        """
        _response = self._client_wrapper.httpx_client.request(
            "eligibility-checks/v1/recommendation",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "filters": filters,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[EligibilityRecommendation],
                parse_obj_as(
                    type_=typing.List[EligibilityRecommendation],  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_recommendation(
        self, *, request: PostEligibilityRecommendationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[EligibilityRecommendation]:
        """
        Create an eligibiilty recommendation based on the request.

        Parameters
        ----------
        request : PostEligibilityRecommendationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EligibilityRecommendation]
        """
        _response = self._client_wrapper.httpx_client.request(
            "eligibility-checks/v1/recommendation",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EligibilityRecommendation,
                parse_obj_as(
                    type_=EligibilityRecommendation,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def vote_recommendation(
        self,
        recommendation_id: str,
        version: str,
        *,
        request: Vote,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EligibilityRecommendation]:
        """
        Submit user feedback on an eligibility recommendation. The path must contain the next version number to prevent race conditions. For example, if the current version of the recommendation is n, you will need to send a request to this endpoint with `/{recommendation_id}/{n+1}/vote` to update the vote.

        Parameters
        ----------
        recommendation_id : str

        version : str

        request : Vote

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EligibilityRecommendation]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"eligibility-checks/v1/recommendation/{jsonable_encoder(recommendation_id)}/{jsonable_encoder(version)}/vote",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EligibilityRecommendation,
                parse_obj_as(
                    type_=EligibilityRecommendation,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_multi(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        subscriber_member_id: typing.Optional[str] = None,
        payer_id: typing.Optional[str] = None,
        provider_npi: typing.Optional[str] = None,
        date_of_service: typing.Optional[str] = None,
        initiated_at_min: typing.Optional[dt.datetime] = None,
        initiated_at_max: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EligibilityCheckPage]:
        """
        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]

        subscriber_member_id : typing.Optional[str]

        payer_id : typing.Optional[str]

        provider_npi : typing.Optional[str]

        date_of_service : typing.Optional[str]

        initiated_at_min : typing.Optional[dt.datetime]

        initiated_at_max : typing.Optional[dt.datetime]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EligibilityCheckPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "eligibility-checks/v1/get-multi/",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "page_token": page_token,
                "limit": limit,
                "subscriber_member_id": subscriber_member_id,
                "payer_id": payer_id,
                "provider_npi": provider_npi,
                "date_of_service": date_of_service,
                "initiated_at_min": serialize_datetime(initiated_at_min) if initiated_at_min is not None else None,
                "initiated_at_max": serialize_datetime(initiated_at_max) if initiated_at_max is not None else None,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EligibilityCheckPage,
                parse_obj_as(
                    type_=EligibilityCheckPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def post(
        self, *, request: EligibilityRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[EligibilityResponse]:
        """
        Sends real-time eligibility checks to payers through Stedi.

        Parameters
        ----------
        request : EligibilityRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EligibilityResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "eligibility-checks/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EligibilityResponse,
                parse_obj_as(
                    type_=EligibilityResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def batch(
        self, *, request: typing.Sequence[EligibilityRequest], request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[BatchEligibilityResponse]:
        """
        Sends a batch of eligibility checks to payers through Stedi.

        Parameters
        ----------
        request : typing.Sequence[EligibilityRequest]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BatchEligibilityResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "eligibility-checks/v1/batch",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                BatchEligibilityResponse,
                parse_obj_as(
                    type_=BatchEligibilityResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def poll_batch(
        self,
        batch_id: str,
        *,
        page_token: typing.Optional[PageToken] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EligibilityCheckPage]:
        """
        Polls the status of a batch eligibility check.

        Parameters
        ----------
        batch_id : str

        page_token : typing.Optional[PageToken]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EligibilityCheckPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"eligibility-checks/v1/batch/{jsonable_encoder(batch_id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "page_token": page_token,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EligibilityCheckPage,
                parse_obj_as(
                    type_=EligibilityCheckPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def payer_search(
        self,
        *,
        page_size: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        query: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayerSearchResponse]:
        """
        Searches for payers that match the query parameters.

        Parameters
        ----------
        page_size : typing.Optional[int]

        page_token : typing.Optional[PageToken]

        query : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayerSearchResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "eligibility-checks/v1/payer/search",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "page_size": page_size,
                "page_token": page_token,
                "query": query,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                PayerSearchResponse,
                parse_obj_as(
                    type_=PayerSearchResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def recommendation(
        self,
        *,
        filters: typing.Optional[FilterQueryString] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.List[EligibilityRecommendation]]:
        """
        Gets recommendation for eligibility checks based on filters. This endpoint will retrieve all the latest eligibility recommendations for each
        eligibility recommendation type for the given filters. If you want to get a specific recommendation type, you can use the `type` query parameter.

        Parameters
        ----------
        filters : typing.Optional[FilterQueryString]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[EligibilityRecommendation]]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "eligibility-checks/v1/recommendation",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "filters": filters,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[EligibilityRecommendation],
                parse_obj_as(
                    type_=typing.List[EligibilityRecommendation],  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_recommendation(
        self, *, request: PostEligibilityRecommendationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[EligibilityRecommendation]:
        """
        Create an eligibiilty recommendation based on the request.

        Parameters
        ----------
        request : PostEligibilityRecommendationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EligibilityRecommendation]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "eligibility-checks/v1/recommendation",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EligibilityRecommendation,
                parse_obj_as(
                    type_=EligibilityRecommendation,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def vote_recommendation(
        self,
        recommendation_id: str,
        version: str,
        *,
        request: Vote,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EligibilityRecommendation]:
        """
        Submit user feedback on an eligibility recommendation. The path must contain the next version number to prevent race conditions. For example, if the current version of the recommendation is n, you will need to send a request to this endpoint with `/{recommendation_id}/{n+1}/vote` to update the vote.

        Parameters
        ----------
        recommendation_id : str

        version : str

        request : Vote

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EligibilityRecommendation]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"eligibility-checks/v1/recommendation/{jsonable_encoder(recommendation_id)}/{jsonable_encoder(version)}/vote",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EligibilityRecommendation,
                parse_obj_as(
                    type_=EligibilityRecommendation,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_multi(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        subscriber_member_id: typing.Optional[str] = None,
        payer_id: typing.Optional[str] = None,
        provider_npi: typing.Optional[str] = None,
        date_of_service: typing.Optional[str] = None,
        initiated_at_min: typing.Optional[dt.datetime] = None,
        initiated_at_max: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EligibilityCheckPage]:
        """
        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]

        subscriber_member_id : typing.Optional[str]

        payer_id : typing.Optional[str]

        provider_npi : typing.Optional[str]

        date_of_service : typing.Optional[str]

        initiated_at_min : typing.Optional[dt.datetime]

        initiated_at_max : typing.Optional[dt.datetime]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EligibilityCheckPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "eligibility-checks/v1/get-multi/",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "page_token": page_token,
                "limit": limit,
                "subscriber_member_id": subscriber_member_id,
                "payer_id": payer_id,
                "provider_npi": provider_npi,
                "date_of_service": date_of_service,
                "initiated_at_min": serialize_datetime(initiated_at_min) if initiated_at_min is not None else None,
                "initiated_at_max": serialize_datetime(initiated_at_max) if initiated_at_max is not None else None,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EligibilityCheckPage,
                parse_obj_as(
                    type_=EligibilityCheckPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.datetime_utils import serialize_datetime
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from ....commons.errors.bad_request_error import BadRequestError
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.internal_error import InternalError
from ....commons.types.bad_request_error_message import BadRequestErrorMessage
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.internal_error_message import InternalErrorMessage
from ....commons.types.page_token import PageToken
from .types.event import Event
from .types.event_id import EventId
from .types.event_scan_page import EventScanPage


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def scan(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        event_types: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        created_before: typing.Optional[dt.datetime] = None,
        created_after: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EventScanPage]:
        """
        Scans the last 30 days of events. All results are sorted by created date, descending.

        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]
            Number of events to return. Minimum value is 1, maximum is 100. Defaults to 10.

        event_types : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Event types to filter on. Defaults to showing all event types.

            | Event Type | Description |
            |------------|-------------|
            | `CandidApi.Encounter.Created` | When a new claim is created |
            | `CandidApi.Encounter.StatusUpdated` | When a claim's status has been updated |
            | `PreEncounter.Patient.Created` | When a new patient has been created |
            | `PreEncounter.Patient.Updated` | When a patient's information has been updated |
            | `PreEncounter.Coverage.Created` | When a new coverage has been created for a patient |
            | `PreEncounter.Coverage.Updated` | When a patient's coverage has been updated |
            | `PreEncounter.Tag.Created` | When a new tag has been created for a patient |
            | `PreEncounter.Tag.Updated` | When a patient's tag has been updated |

        created_before : typing.Optional[dt.datetime]
            Filters for only events created before this time (inclusive).

        created_after : typing.Optional[dt.datetime]
            Filters for only events created after this time (inclusive).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EventScanPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/events/v1/",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "page_token": page_token,
                "limit": limit,
                "event_types": event_types,
                "created_before": serialize_datetime(created_before) if created_before is not None else None,
                "created_after": serialize_datetime(created_after) if created_after is not None else None,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EventScanPage,
                parse_obj_as(
                    type_=EventScanPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequestError":
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequestErrorMessage,
                        parse_obj_as(
                            type_=BadRequestErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InternalError":
                raise InternalError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InternalErrorMessage,
                        parse_obj_as(
                            type_=InternalErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(self, event_id: EventId, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[Event]:
        """
        Parameters
        ----------
        event_id : EventId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Event]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/events/v1/{jsonable_encoder(event_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Event,
                parse_obj_as(
                    type_=Event,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequestError":
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequestErrorMessage,
                        parse_obj_as(
                            type_=BadRequestErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InternalError":
                raise InternalError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InternalErrorMessage,
                        parse_obj_as(
                            type_=InternalErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def scan(
        self,
        *,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        event_types: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        created_before: typing.Optional[dt.datetime] = None,
        created_after: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EventScanPage]:
        """
        Scans the last 30 days of events. All results are sorted by created date, descending.

        Parameters
        ----------
        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]
            Number of events to return. Minimum value is 1, maximum is 100. Defaults to 10.

        event_types : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Event types to filter on. Defaults to showing all event types.

            | Event Type | Description |
            |------------|-------------|
            | `CandidApi.Encounter.Created` | When a new claim is created |
            | `CandidApi.Encounter.StatusUpdated` | When a claim's status has been updated |
            | `PreEncounter.Patient.Created` | When a new patient has been created |
            | `PreEncounter.Patient.Updated` | When a patient's information has been updated |
            | `PreEncounter.Coverage.Created` | When a new coverage has been created for a patient |
            | `PreEncounter.Coverage.Updated` | When a patient's coverage has been updated |
            | `PreEncounter.Tag.Created` | When a new tag has been created for a patient |
            | `PreEncounter.Tag.Updated` | When a patient's tag has been updated |

        created_before : typing.Optional[dt.datetime]
            Filters for only events created before this time (inclusive).

        created_after : typing.Optional[dt.datetime]
            Filters for only events created after this time (inclusive).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EventScanPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/events/v1/",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "page_token": page_token,
                "limit": limit,
                "event_types": event_types,
                "created_before": serialize_datetime(created_before) if created_before is not None else None,
                "created_after": serialize_datetime(created_after) if created_after is not None else None,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EventScanPage,
                parse_obj_as(
                    type_=EventScanPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequestError":
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequestErrorMessage,
                        parse_obj_as(
                            type_=BadRequestErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InternalError":
                raise InternalError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InternalErrorMessage,
                        parse_obj_as(
                            type_=InternalErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, event_id: EventId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Event]:
        """
        Parameters
        ----------
        event_id : EventId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Event]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/events/v1/{jsonable_encoder(event_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Event,
                parse_obj_as(
                    type_=Event,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequestError":
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequestErrorMessage,
                        parse_obj_as(
                            type_=BadRequestErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InternalError":
                raise InternalError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InternalErrorMessage,
                        parse_obj_as(
                            type_=InternalErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....common.types.organization_id import OrganizationId
from .....common.types.patient_id import PatientId


class EligibilityRecommendationPatientInfo(UniversalBaseModel):
    """
    An object representing patient information for an eligibility recommendation.
    This is used to find recommendations. Each field helps us find the right corresponding
    eligibility recommendation for the patient.
    """

    id: typing.Optional[PatientId] = None
    mrn: typing.Optional[str] = None
    organization_id: typing.Optional[OrganizationId] = None
    last_name: typing.Optional[str] = None
    first_name: typing.Optional[str] = None
    date_of_birth: typing.Optional[dt.date] = None
    member_id: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core import enum

T_Result = typing.TypeVar("T_Result")


class PatientTransactionSource(enum.StrEnum):
    MANUAL_ENTRY = "MANUAL_ENTRY"
    CHARGEBEE = "CHARGEBEE"
    SQUARE = "SQUARE"
    STRIPE = "STRIPE"
    ELATION = "ELATION"
    CEDAR = "CEDAR"
    HEALTHIE = "HEALTHIE"
    REALLOCATION = "REALLOCATION"
    PHREESIA = "PHREESIA"
    INSTAMED = "INSTAMED"
    _UNKNOWN = "__PATIENTTRANSACTIONSOURCE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "PatientTransactionSource":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        manual_entry: typing.Callable[[], T_Result],
        chargebee: typing.Callable[[], T_Result],
        square: typing.Callable[[], T_Result],
        stripe: typing.Callable[[], T_Result],
        elation: typing.Callable[[], T_Result],
        cedar: typing.Callable[[], T_Result],
        healthie: typing.Callable[[], T_Result],
        reallocation: typing.Callable[[], T_Result],
        phreesia: typing.Callable[[], T_Result],
        instamed: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is PatientTransactionSource.MANUAL_ENTRY:
            return manual_entry()
        if self is PatientTransactionSource.CHARGEBEE:
            return chargebee()
        if self is PatientTransactionSource.SQUARE:
            return square()
        if self is PatientTransactionSource.STRIPE:
            return stripe()
        if self is PatientTransactionSource.ELATION:
            return elation()
        if self is PatientTransactionSource.CEDAR:
            return cedar()
        if self is PatientTransactionSource.HEALTHIE:
            return healthie()
        if self is PatientTransactionSource.REALLOCATION:
            return reallocation()
        if self is PatientTransactionSource.PHREESIA:
            return phreesia()
        if self is PatientTransactionSource.INSTAMED:
            return instamed()
        return _unknown_member(self._value_)
